==> Catmandu-XML-0.13.tar.gz: Catmandu-XML-0.13/lib/Catmandu/Importer/XML.pm <==
package Catmandu::Importer::XML;
# ABSTRACT: Import serialized XML documents
our $VERSION = '0.13'; # VERSION

use namespace::clean;
use Catmandu::Sane;
use Moo;
use XML::Struct::Reader;

with 'Catmandu::Importer';

has type        => (is => 'ro', default => sub { 'simple' });
has path        => (is => 'ro');
has root        => (is => 'lazy', default => sub { defined $_[0]->path ? 1 : 0 });
has depth       => (is => 'ro');
has ns          => (is => 'ro', default => sub { '' });
has attributes  => (is => 'ro', default => sub { 1 });
has whitespace  => (is => 'ro', default => sub { 0 });

sub generator {
    my ($self) = @_;
    sub {
        state $reader = do { 
            my %options = (
                from       => ($self->file || $self->fh),
                whitespace => $self->whitespace,
                attributes => $self->attributes,
                depth      => $self->depth,
                ns         => $self->ns,
            );
            $options{path} = $self->path if defined $self->path;
            if ($self->type eq 'simple') {
                $options{simple} = 1;
                $options{root} = $self->root;
            } elsif ($self->type ne 'ordered') {
                return;
            }
            XML::Struct::Reader->new(%options);
        };

        return $reader->readNext;
    }
}


1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Catmandu::Importer::XML - Import serialized XML documents

=head1 VERSION

version 0.13

=head1 DESCRIPTION

This importer reads XML and transforms it into a data structure. 

=head1 CONFIGURATION

=over 4

=item type

By default (type "C<simple>"), elements and attributes and converted to keys in
a key-value structure. For instance this document: 

    <doc attr="value">
      <field1>foo</field1>
      <field1>bar</field1>
      <bar>
        <doz>baz</doz>
      </bar>
    </doc>

is imported as

    {
        attr => 'value',
        field1 => [ 'foo', 'bar' ],
        field2 => { 'doz' => 'baz' },
    }

With type "C<ordered>" elements are preserved in the order of their appereance.
For instance the sample document above is imported as:

    [ 
        doc => { attr => "value" }, [
            [ field1 => { }, ["foo"] ],
            [ field1 => { },  ["bar"] ],
            [ field2 => { }, [ [ doz => { }, ["baz"] ] ] ]
        ]
    ] 

=item depth

Maximum depth for type "C<simple>". For instance with depth 1, the sample document above
would be imported as:

    {
        attr => 'value',
        field1 => [ 'foo', 'bar' ],
        field2 => { 
            doz => [ [ doz => { }, ["baz"] ] ]
        }
    }

=item attributes

Include XML attributes. Enabled by default.

=item path

Path expression to select XML elements. If not set the root element is
selected.

=item root

Include root element name, if enabled. Disabled by default, unless the C<path>
option is set.

=item ns

Set to 'C<strip>' for stripping namespace prefixes and xmlns-attributes.

=item whitespace

Include ignoreable whitespace. Disabled by default.

=back

=head1 SEE ALSO

This module is just a thin layer on top of L<XML::Struct::Reader>. Have a look
at L<XML::Struct> to implement Importers and Exporters for more specific
XML-based data formats.

=head1 AUTHOR

Jakob Voß

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2014 by Jakob Voß.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut

==> Catmandu-XML-0.14.tar.gz: Catmandu-XML-0.14/lib/Catmandu/Importer/XML.pm <==
package Catmandu::Importer::XML;
# ABSTRACT: Import serialized XML documents
our $VERSION = '0.14'; # VERSION

use namespace::clean;
use Catmandu::Sane;
use Moo;
use XML::Struct::Reader;
use Catmandu::XML::Transformer;

with 'Catmandu::Importer';

has type        => (is => 'ro', default => sub { 'simple' });
has path        => (is => 'ro');
has root        => (is => 'lazy', default => sub { defined $_[0]->path ? 1 : 0 });
has depth       => (is => 'ro');
has ns          => (is => 'ro', default => sub { '' });
has attributes  => (is => 'ro', default => sub { 1 });
has whitespace  => (is => 'ro', default => sub { 0 });
has transform   => (is => 'ro', coerce => sub { 
        Catmandu::XML::Transformer->new( stylesheet => $_[0] ) });

sub generator {
    my ($self) = @_;
    sub {
        state $reader = do { 
            my %options = (
                from       => ($self->file || $self->fh),
                whitespace => $self->whitespace,
                attributes => $self->attributes,
                depth      => $self->depth,
                ns         => $self->ns,
            );
            $options{path} = $self->path if defined $self->path;
            if ($self->type eq 'simple') {
                $options{simple} = 1;
                $options{root} = $self->root;
            } elsif ($self->type ne 'ordered') {
                return;
            }
            XML::Struct::Reader->new(%options);
        };
    
        my $item = $reader->readNext;

        # TODO: transformation should be done earlier for efficiency
        # and because simple format modifies the XML document (bug)
        return $self->transform ?
               $self->transform->transform($item) : $item
    }
}


1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Catmandu::Importer::XML - Import serialized XML documents

=head1 VERSION

version 0.14

=head1 DESCRIPTION

This L<Catmandu::Importer> reads XML and transforms it into a data structure. 

See L<Catmandu::Importer>, L<Catmandu::Iterable>, L<Catmandu::Logger> and
L<Catmandu::Fixable> for methods and options derived from these modules.

The importer can also be used internally for custom importers that need to
parse XML data.

=head1 CONFIGURATION

=over

=item type

By default (type "C<simple>"), elements and attributes and converted to keys in
a key-value structure. For instance this document: 

    <doc attr="value">
      <field1>foo</field1>
      <field1>bar</field1>
      <bar>
        <doz>baz</doz>
      </bar>
    </doc>

is imported as

    {
        attr => 'value',
        field1 => [ 'foo', 'bar' ],
        field2 => { 'doz' => 'baz' },
    }

With type "C<ordered>" elements are preserved in the order of their appereance.
For instance the sample document above is imported as:

    [ 
        doc => { attr => "value" }, [
            [ field1 => { }, ["foo"] ],
            [ field1 => { },  ["bar"] ],
            [ field2 => { }, [ [ doz => { }, ["baz"] ] ] ]
        ]
    ] 

=item depth

Maximum depth for type "C<simple>". For instance with depth 1, the sample document above
would be imported as:

    {
        attr => 'value',
        field1 => [ 'foo', 'bar' ],
        field2 => { 
            doz => [ [ doz => { }, ["baz"] ] ]
        }
    }

=item attributes

Include XML attributes. Enabled by default.

=item path

Path expression to select XML elements. If not set the root element is
selected.

=item root

Include root element name, if enabled. Disabled by default, unless the C<path>
option is set.

=item ns

Set to C<strip> for stripping namespace prefixes and xmlns-attributes.

=item whitespace

Include ignoreable whitespace. Disabled by default.

=item transform

Optional (list of) XSLT stylesheets to process records. This option is not
fully implemented yet!

=back

=head1 SEE ALSO

This module is just a thin layer on top of L<XML::Struct::Reader>. Have a look
at L<XML::Struct> to implement Importers and Exporters for more specific
XML-based data formats.

=head1 AUTHOR

Jakob Voß

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2014 by Jakob Voß.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
